###############################################################################
# Copyright (c) 2011, 2011 Linaro
# All rights reserved. This program and the accompanying materials
# are made available under the terms of the Eclipse Public License v1.0
# which accompanies this distribution, and is available at
# http://www.eclipse.org/legal/epl-v10.html
#
# Contributors:
#     Linaro Infrastructure Team - initial implementation
###############################################################################
import re
import io
import gzip
import urllib
from StringIO import StringIO
from collections import deque
from config import print_message


class MatcherError(Exception):
    def __init__(self, value):
        self.value = value

    def __str__(self):
        return repr(self.value)


def get_file_contents(gz_url):
    """Return the contents of gzip file at url."""

    file = ''
    if gz_url is not None:
        file_handle = urllib.urlopen(gz_url)
        if file_handle is not None:
            file_gzip = file_handle.read()
            file_io = io.BytesIO(file_gzip)
            if file_io is not None:
                try:
                    gz_file = gzip.GzipFile(fileobj=file_io)
                    if gz_file is not None:
                        file = unicode(gz_file.read(), errors='replace')
                        gz_file.close()
                except:
                    # Log is not compressed, might be 404 error.
                    # TODO: should this be analyzed or just thrown away?
                    file = ''
                file_io.close()
            file_handle.close()
    return file


def get_interesting_part(text):

    class AbstractMatcher(object):
        name = ''

        def __init__(self, name):
            self.name = name

        def match(self, string):
            raise NotImplementedError('match method not implemented')

        def get_name(self):
            return self.name

    class RegexpMatcher(AbstractMatcher):

        def __init__(self, name, regexp):
            super(RegexpMatcher, self).__init__(name)
            self.regexp = regexp

        def match(self, string):
            m = re.search(self.regexp, string)
            if m is not None:
                return True
            else:
                return False

    class ExactMatcher(AbstractMatcher):

        def __init__(self, name, string):
            super(ExactMatcher, self).__init__(name)
            self.string = string

        def match(self, string):
            return self.string == string

    matchers = [
#     ExactMatcher('Kernel image error', 'make: *** [kernel-image] Error 2\n'),
        RegexpMatcher('Purging chroot-autobuild',
                      '^Purging chroot-autobuild/'),
        RegexpMatcher('Source-dependencies not satisfied',
                      '^Source-dependencies not satisfied'),
        ]

    history = deque([])
    history_max = 20
    skipped_lines = 0
    buffer = StringIO(text)
    for line in buffer:
        history.append(line)
        if len(history) > history_max - 1:
            history.popleft()
            skipped_lines += 1
        for matcher in matchers:
            if isinstance(matcher, AbstractMatcher):
                if matcher.match(line):
                    return """This log snippet might be of interest, since it \
triggered the matcher '%s'.\nExcerpt %d lines into the build log:\n\n%s""" % \
                        (matcher.get_name(), skipped_lines, "".join(history))
            else:
                raise MatcherError('Matcher not implemented correctly.')
    return ''
