###############################################################################
# Copyright (c) 2011, 2011 Linaro
# All rights reserved. This program and the accompanying materials
# are made available under the terms of the Eclipse Public License v1.0
# which accompanies this distribution, and is available at
# http://www.eclipse.org/legal/epl-v10.html
#
# Contributors:
#     Linaro Infrastructure Team - initial implementation
###############################################################################
from launchpadlib.launchpad import Launchpad, EDGE_SERVICE_ROOT
from launchpadlib.errors import HTTPError

from launchpadlib.uris import *
import apt_pkg

from config import (
    get_base_url,
    get_base_api_url,
    get_launchpad,
    print_message,
    )

apt_pkg.InitSystem()

# Uncomment for tracing LP API calls
#import httplib2
#httplib2.debuglevel = 1


PRIMARY_ARCHIVE_NAME = 'primary'
FAILED_BUILD_STATE = 'Failed to build'


class InvalidArchiveError(Exception):
    def __init__(self, value):
        self.value = value

    def __str__(self):
        return repr(self.value)


def get_tags_for_fail(fail_data=''):
    """ Determine cause of failure and tag accordingly. """

    tags = []
    if fail_data != '':
        # Process build log contents to determine extra tags
        pass
    return tags


class SPPH(object):
    def __init__(self, spph):
        self.spph = spph
        self.logs = dict()
        self.version = self.spph.source_package_version
        self.package_name = self.spph.source_package_name
        self.url = get_base_api_url() + '+source/%s' % self.package_name

    def add_build_log(self, buildlog):
        """Adds build log if buildlog is the most recent build yet seen
        for the respective arch.
        """
        stored_build = self.get_arch(buildlog.arch_tag)
        if (stored_build is None or
            buildlog.datecreated > stored_build.datecreated):
            self.logs[buildlog.arch_tag] = buildlog

    def get_arch(self, arch):
        return self.logs.get(arch)


def fetch_pkg_list(archives):
    all_spph = dict()
    for archive, archive_name in archives:
        print_message("The following packages have failed to build " \
                      "in '%s'." % archive_name)

        buildlist = archive.getBuildRecords(build_state=FAILED_BUILD_STATE)

        for build in buildlist:
            csp = build.current_source_publication
            if csp is None:
                # Build log for an older version
                continue

            print_message("  Found build '%s'" % build.title)

            spph = all_spph.get(csp.source_package_name)
            if spph is None:
                spph = all_spph[csp.source_package_name] = SPPH(csp)

            spph.add_build_log(build)

    return all_spph


def filter_spph(all_spph_list, fail_arch, ok_archs):
    """Filter out the build failures that are unique to the target arch.

    Arguments:
    all_spph_list -- List of all spphs to investigate.
    fail_arch -- The arch on which the builds should have failed.
    ok_archs -- A list of archs on which the builds have not failed.

    Returns:
    A subset of all_spph_list which represents the interesting packages.
    """
    filtered_list = []
    for spph in all_spph_list.values():
        if spph.get_arch(fail_arch) is None:
            # Drop this spph since it did not fail on target arch.
            continue
        if next((arch for arch in ok_archs if spph.get_arch(arch)),
                None) is not None:
            # Drop this spph since it did fail on other arch.
            continue
        filtered_list.append(spph)

    return filtered_list


def get_build_status(archive_names, series_name, fail_platform, ok_platforms):
    """Download live data from lp.

    Parameters:
    archive_names - List of archives. Eg ['primary', 'rebuild-x']
    series_name - Name of series. Eg 'natty'
    fail_platform - Name of target platform.
    ok_platforms - List of names of platforms which shouldn't have failed.

    Returns:
    A list of SPPH objects representing source packages to file bugs against.
    """

    ubuntu = get_launchpad().distributions['ubuntu']

    archives = []
    series = None
    for archive_name in archive_names:
        try:
            archive = None
            archive_display_name = ''
            if archive_name == PRIMARY_ARCHIVE_NAME:
            # We only filter by series if we're looking in the main archive.
                if series_name is None:
                    archive = ubuntu.current_series
                else:
                    archive = ubuntu.getSeries(name_or_version=series_name)
                archive_display_name = PRIMARY_ARCHIVE_NAME + '/'
            else:
                archive = ubuntu.getArchive(name=archive_name)
            if archive is None:
                raise InvalidArchiveError(
                    'Error: %s is not a valid archive.' % archive_name)
            archive_display_name = archive_display_name + archive.name
            archives.append((archive, archive_display_name))
        except HTTPError:
            print 'Error: %s is not a valid archive.' % archive_name
            raise

    print_message("Will be getting builds from archives " + ", ".join(
            name for (archive, name) in archives))
    spph_list = fetch_pkg_list(archives)
    return filter_spph(spph_list, fail_platform, ok_platforms)
