###############################################################################
# Copyright (c) 2011, 2011 Linaro
# All rights reserved. This program and the accompanying materials
# are made available under the terms of the Eclipse Public License v1.0
# which accompanies this distribution, and is available at
# http://www.eclipse.org/legal/epl-v10.html
#
# Contributors:
#     Linaro Infrastructure Team - initial implementation
###############################################################################
import __builtin__
from testtools import TestCase

from tests.fixtures import (
    TestCaseWithFixtures,
    MockSomethingFixture,
    )
from logging import (
    LOG_ITEM_SEPARATOR,
    Log,
    LogEntry,
    )


class TestLogEntry(TestCase):

    def setUp(self):
        super(TestLogEntry, self).setUp()

    def test_log_line_format(self):
        package_name = 'package'
        package_version = '1.0.1'
        build_arch = 'armel'
        bugtask_url = 'http://dummy'
        log_entry = LogEntry(package_name, package_version, build_arch,
                                    bugtask_url)
        self.assertEquals(log_entry.to_string(),
                          package_name + LOG_ITEM_SEPARATOR + package_version +
                          LOG_ITEM_SEPARATOR + build_arch +
                          LOG_ITEM_SEPARATOR + bugtask_url)

    def test_log_line_matches(self):
        package_name = 'package'
        package_version = '1.0.1'
        build_arch = 'armel'
        bugtask_url = 'http://dummy'
        log_entry = LogEntry(package_name, package_version, build_arch,
                             bugtask_url)
        self.assertTrue(log_entry.matches(package_name, package_version,
                                          build_arch))

    def test_log_line_does_not_match_other_version(self):
        package_name = 'package'
        package_version_logged = '1.0.1'
        package_version_to_match = '2'
        build_arch = 'armel'
        bugtask_url = 'http://dummy'
        log_entry = LogEntry(package_name, package_version_logged, build_arch,
                             bugtask_url)
        self.assertFalse(
            log_entry.matches(package_name, package_version_to_match,
                              build_arch))


class TestLog(TestCaseWithFixtures):

    def setUp(self):
        super(TestLog, self).setUp()

    def test_add_entry_does_so(self):
        package_name = 'package'
        package_version = '1.0.1'
        build_arch = 'armel'
        bugtask_url = 'http://dummy'
        log_entry = LogEntry(package_name, package_version, build_arch,
                             bugtask_url)
        log = Log()
        log.add_entry(log_entry)
        self.assertIn(log_entry, log.log_entries)

    def test_get_nonexistent_logs_fails(self):
        log = Log()
        self.assertEquals(None, log.get_entry('package', 'version', 'arch'))

    def test_stored_log_is_found(self):
        package_name = 'package'
        package_version = '1.0.1'
        build_arch = 'armel'
        bugtask_url = 'http://dummy'
        log_entry = LogEntry(package_name, package_version, build_arch,
                             bugtask_url)
        log = Log()
        log.add_entry(log_entry)
        self.assertEquals(
            log_entry, log.get_entry(package_name, package_version,
                                     build_arch))

    def test_no_entries_in_log_without_file(self):
        log = Log(None)
        self.assertEquals([], log.log_entries)

    def test_no_entries_in_log_with_empty_file(self):
        log_file = self.createTempFileAsFixture()
        log = Log(log_file)
        self.assertEquals([], log.log_entries)

    def test_read_log_entry_from_file(self):
        package_name = 'packagename'
        package_version = '1.1.2'
        build_arch = 'armel'
        bugtask_url = 'http://dummy'
        log_entry = LogEntry(package_name, package_version, build_arch,
                             bugtask_url)
        log_file_name = self.createTempFileAsFixture()
        with open(log_file_name, 'w') as log_file_write:
            log_file_write.write(log_entry.to_string())

        log = Log(log_file_name)
        log_entry_read = log.get_entry(package_name, package_version,
                                       build_arch)
        self.assertTrue(len(log.log_entries) == 1)
        self.assertTrue(log_entry_read.matches(
                package_name, package_version, build_arch))

    def test_write_log_entry_to_file(self):
        package_name = 'packagename'
        package_version = '1.2.3-1'
        build_arch = 'armel'
        bugtask_url = 'http://dummy'
        log_entry = LogEntry(package_name, package_version, build_arch,
                             bugtask_url)
        log_file_name = self.createTempFileAsFixture()
        log_write = Log(log_file_name)
        log_write.write_entry(log_entry.to_string())
        del log_write
        with open(log_file_name, 'r') as log_file:
            log_line = log_file.readline().rstrip('\n')
        self.assertEquals(log_entry.to_string(), log_line)

    def test_failed_open_log_for_writing_raises(self):
        def mock_open(self, string):
            raise IOError("Test error.")

        def create_log(file_name):
            log_write = Log(file_name)

        self.useFixture(MockSomethingFixture(
                __builtin__, 'open', mock_open))
        log_file_name = 'logfile'
        self.assertRaises(IOError, create_log, log_file_name)
